
/* PAEE engine for REX 6000/DATASLIM 2         */
/* (C) Copyright 2010 Enrique D. Bosch 'presi' */

/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License    */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

/* Compile with rexdk, memoio-ro lib required  */


#include <rex/rex.h>
#include <stdlib.h>
#include <memoio.h>

#define OFFSET   MEMO_SEEK_SET

#define TRUE     1
#define FALSE    0

#define NORMAL   0
#define MINI     0x20

#define L_TBOT   50
#define L_LIN    50

typedef char byte;
typedef char bool;

unsigned int g_possig[9];
bool g_seguir;


void echo(char *texto, bool reset)
{
  static byte y;

  if (reset) { y = 0; return; }    

  DsPrintf(0,y,NORMAL,texto);
  y += 12;
}

void boton(char *texto, byte c)
{
  DsDialogTextButton(70,108,100,12,1,"");
  DsPrintf(c,110,MINI,texto);
}

void boton_opcion(byte code, byte lopc, bool reset)
{
  static byte y;
  byte alto;

  if (reset) { y = 0; return; }

  alto = 12*lopc+4;
  DsDialogTextButton(0,y,240,alto,code,"");
  y += alto+2;
}

void texto_boton(char *texto, byte op, bool reset)
{
  static byte y, opa;
  
  if (reset) { y = 2; opa = 0; return; }

  if (opa!=op) { y += 6; opa = op; }
  DsPrintf(3,y,NORMAL,texto);
  y += 12;
}

void cls(void)
{
  DsEventClear();
  DsClearScreen();
  echo("",TRUE);
}

void clb(void)
{
  boton_opcion(0,0,TRUE);
  texto_boton("",0,TRUE);
}

void memo_gets_lin(char *buffer, unsigned int length, MEMO *stream)
{
  unsigned int i;
  char ch;

  for (i = 0; i<length; i++)
  {
    ch = memo_getc(stream);
    if (ch==10) break;
    *buffer = ch;
    buffer++;
  }
  *buffer = 0;
}

unsigned int memo_gets_int(MEMO *stream)
{
  char num[9];
  
  memo_gets_lin(num,6,stream);
  return atoi(num);
}

byte input(MEMO *stream)
{
  MSG msg;
  bool esperar = TRUE;
  byte code;
  
  while (esperar)
  {
    DsEventMessageGet(&msg);
    switch (msg.message)
    {
      case MSG_DS_COMMAND:
         if (msg.sCode==MSG_DS_COMMAND_DOWN) { code = msg.bCode; esperar = FALSE; }
         break;
      case MSG_DS_KEY_DOWN:
         switch (msg.sCode)
         {
           case KEY_TOP_C:    /* salir */
              cls();
              memo_close(stream);
              DsAddinTerminate();
           case KEY_BACK_A:   /* volver a empezar */
              code = 0;
              esperar = FALSE;
              break;
           case KEY_ENTER_A:  /* seguir */
           case KEY_DOWN_A:
              code = 1;
              esperar = FALSE;
              break;
           case KEY_UP_A:     /* releer */
              code = 2;
              esperar = FALSE;
              break;
         } 
    }
  }
  return code;
}

byte memo_jump(MEMO *stream, char *texto, byte c, bool opc)
{
  unsigned int next;
  byte in;

  boton(texto,c);
  in = input(stream);
  if (opc && in==1) return 0;
  switch (in)
  {
    case 1:  next = memo_gets_int(stream);
             break;
    case 2:  next = g_possig[2];
             g_seguir = TRUE;
             break;
    default: next = 0;
             g_seguir = TRUE;
  }
  memo_seek(stream,next,OFFSET);
  return 1;
}

void memo_handle_options(MEMO *stream)
{
  byte i, j, numopc, lopc, imt, ind, code;
  char buffer[L_TBOT+2];

  code = memo_jump(stream,"Mostrar opciones",82,TRUE);
  if (code) return;
  numopc = memo_gets_int(stream);
  cls();
  clb();
  for (i = 0, imt = 3; i<numopc; i++, imt++)
  {
    lopc = memo_gets_int(stream);
    memo_getc(stream);
    g_possig[imt] = memo_gets_int(stream);
    boton_opcion(imt,lopc,FALSE);
    for (j = 0; j<lopc; j++)
    {
      memo_gets_lin(buffer,L_TBOT,stream);
      texto_boton(buffer,i,FALSE);
    }
  }
  while ((ind = input(stream))==1);
  memo_seek(stream,g_possig[ind],OFFSET);
  if (ind) g_possig[2] = g_possig[ind];
  else g_possig[2] = 0;
}


int main(void)
{
  MEMO paee;
  byte i, lins;
  char buffer[L_LIN+2];

  g_possig[0] = 0;
  g_possig[2] = 0;
  g_seguir = TRUE;
  if ( memo_open(&paee,"zpaee","rb")<0 )
    { cls(); DsPrintf(20,54,NORMAL,"Por favor, carga el memo zpaee");
      input(0); cls(); return 1; }

  while (g_seguir)
  {
    cls();
    lins = memo_gets_int(&paee);
    for (i = 0; i<lins; i++)
    {
      memo_gets_lin(buffer,L_LIN,&paee);
      echo(buffer,FALSE);
    }
    switch (memo_getc(&paee))
    {
      case 'C': memo_jump(&paee,"Continuar",100,FALSE);
                break;
      case 'O': memo_handle_options(&paee);
                break;
      case 'F': g_seguir = FALSE;
                memo_jump(&paee,"Fin",115,FALSE);
                break;
    }
  }
  memo_close(&paee);
  return 0;
}
